#!/bin/bash

# uses the following optional variables:
#  - NPROC processes (default: $(nproc))
#  - CC: full path to gcc, default: ${CROSS_COMPILE}gcc
#  - CROSS_COMPILE: prefix to the compiler. default: CROSS_COMPILE_$arch
#  - CROSS_COMPILE_$arch: default for $arch, default: located in $TOOLCHAIN_BASE
#  - TOOLCHAIN_BASE: directory containing the nolibc toolchains
#  - USER_ONLY: if set, doesn't build kernels nor start qemu
#  - V: verbose build, passed through the whole build chain
#
# Arguments: list of archs to build and test
#
NPROC=${NPROC:-$(nproc)}
TOOLCHAIN_BASE=${TOOLCHAIN_BASE:-/f/tc/nolibc/gcc-13.2.0-nolibc}

if [ ! -e nolibc-test.c ]; then
	echo "Please run from the nolibc directory."
	exit 1
fi

export CROSS_COMPILE_i386="${CROSS_COMPILE_i386:-${TOOLCHAIN_BASE}/i386-linux/bin/i386-linux-}"
export CROSS_COMPILE_x86_64="${CROSS_COMPILE_x86_64:-${TOOLCHAIN_BASE}/x86_64-linux/bin/x86_64-linux-}"
export CROSS_COMPILE_arm64="${CROSS_COMPILE_arm64:-${TOOLCHAIN_BASE}/aarch64-linux/bin/aarch64-linux-}"
export CROSS_COMPILE_arm="${CROSS_COMPILE_arm:-${TOOLCHAIN_BASE}/arm-linux-gnueabi/bin/arm-linux-gnueabi-}"
export CROSS_COMPILE_mips="${CROSS_COMPILE_mips:-${TOOLCHAIN_BASE}/mips-linux/bin/mips-linux-}"
export CROSS_COMPILE_ppc="${CROSS_COMPILE_ppc:-${TOOLCHAIN_BASE}/powerpc-linux/bin/powerpc-linux-}"
export CROSS_COMPILE_ppc64="${CROSS_COMPILE_ppc64:-${TOOLCHAIN_BASE}/powerpc64-linux/bin/powerpc64-linux-}"
export CROSS_COMPILE_ppc64le="${CROSS_COMPILE_ppc64le:-${TOOLCHAIN_BASE}/powerpc64-linux/bin/powerpc64-linux-}"
export CROSS_COMPILE_riscv="${CROSS_COMPILE_riscv:-${TOOLCHAIN_BASE}/riscv64-linux/bin/riscv64-linux-}"
export CROSS_COMPILE_s390="${CROSS_COMPILE_s390:-${TOOLCHAIN_BASE}/s390-linux/bin/s390-linux-}"
export CROSS_COMPILE_loongarch="${CROSS_COMPILE_loongarch:-${TOOLCHAIN_BASE}/loongarch64-linux/bin/loongarch64-linux-}"

ARCHS=( "$@" )
if [ "${#ARCHS[@]}" = 0 ]; then
	ARCHS=( i386 x86_64 arm64 arm mips ppc ppc64 ppc64le riscv s390 loongarch )
fi

# stop on first error
set -e

rm -f report.out

for arch in "${ARCHS[@]}"; do
	echo "#### Building for arch $arch ####"

	cross_compile="CROSS_COMPILE_${arch}"
	cross_compile="${CROSS_COMPILE:-${!cross_compile}}"
	cc="${CC:-${cross_compile}gcc}"

	echo "###### CLEAN $arch ######" | tee -a report.out

	make clean XARCH="$arch" \
	     CROSS_COMPILE="$cross_compile" \
	     CC="$cc" ${V:+V=${V}}

	if [ -z "$USER_ONLY" ]; then
		echo "###### CONFIG $arch ######" | tee -a report.out
		
		make -C ../../../.. ARCH="${arch//ppc*/powerpc}" mrproper 
		make defconfig XARCH="$arch" \
		     CROSS_COMPILE="$cross_compile" \
		     CC="$cc" ${V:+V=${V}}
	fi

	echo "###### BUILD $arch (cross $cross_compile) ######" | tee -a report.out

	make -j${NPROC} nolibc-test XARCH="$arch" \
	     CROSS_COMPILE="$cross_compile" \
	     CC="$cc" ${V:+V=${V}} \
	     2>&1 | tee -a report.out

	cp nolibc-test nolibc-test-$arch
	(echo "# size nolibc-test-$arch"; size nolibc-test-$arch) | tee -a report.out

	echo "###### USER $arch ######" | tee -a report.out

	rm -f run.out
	make -j${NPROC} run-user XARCH="$arch" \
	     CROSS_COMPILE="$cross_compile" \
	     CC="$cc" ${V:+V=${V}} \
	     2>&1 | tee -a report.out

	echo "###### RUN-USER LOG for $arch (run.out) ######" | tee -a report.out
	cat run.out >> report.out

	if [ -z "$USER_ONLY" ]; then
		echo "###### KERN $arch ######" | tee -a report.out
	
		rm -f run.out
		make -j${NPROC} run XARCH="$arch" \
		     CROSS_COMPILE="$cross_compile" \
		     CC="$cc" ${V:+V=${V}} \
		     2>&1 | tee -a report.out
	
		echo "###### RUN-KERN LOG for $arch (run.out) ######" | tee -a report.out
		cat run.out >> report.out
	fi
	echo "###### DONE $arch ######" | tee -a report.out
done

echo "#### Synthetic summary:"
grep -o ' USER [^ ]* \| KERN [^ ]* \|.* status:.*' report.out | sed '$!N;s/\n/ /' | column -t
